#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
import wx, wx.adv, sys, os, re
import wx.lib.agw.flatnotebook as fnb
import cv2, numpy as np
import random, math

from optparse import OptionParser

try:
    import winsound

    nosound = False
except ImportError:
    nosound = True
import time


import terasense.config.pixel_config as pc

import terasense.ref as ref
import terasense.processor as  processor
import terasense.worker as worker


import threading

import zipfile



import terasense.gui.bcgpanel as bcgpanel
import terasense.gui.optionspanel as optionspanel



# pixel proportions
pxRATIO = 192.0 / 192.0
pxSCALE = 3



# Button definitions
##ID_START = wx.NewIdRef()
##ID_STOP = wx.NewIdRef()

IMG_PREFIX = "ts_"

MIN_VIDEO_OUT = 16


UpdateEventType = wx.NewEventType()
EVT_UPDATE = wx.PyEventBinder(UpdateEventType, 1)

version = ""
commandline_options = None


class UpdateEvent(wx.PyCommandEvent):
    """Simple event to carry arbitrary result data."""

    def __init__(self, data=None):
        wx.PyCommandEvent.__init__(self, UpdateEventType)
        self.data = data


class ImageDisplay(wx.Panel):
    def __init__(
        self,
        parent,
        id,
        pos=(0, 0),
        size=(400, 400),
        style=0,
        name="",
        imgSize=(16, 16),
    ):

        self.parent = parent

        wx.Panel.__init__(self, parent, id, pos, size, style, name)
        self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)
        self.DCsize = tuple(self.GetClientSize())
        self.size = (imgSize[1],imgSize[0])
        


        try:
            self.bitmap = wx.Bitmap(*self.DCsize)
        except:
            self.bitmap = wx.EmptyBitmap(*self.DCsize)

        self.img = np.zeros(self.DCsize + (3,), dtype=np.uint8)
        self.scaledImg = np.empty(self.DCsize + (3,), dtype=np.uint8)
        self.grid = self.createGrid()

        self.startPos = self.endPos = None

        self.drawGrid = False

        try:
            self.buffer = wx.Bitmap(*self.DCsize)
        except:
            self.buffer = wx.EmptyBitmap(*self.DCsize)

        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Update(self.img)
        try:
            self.SetCursor(wx.Cursor(wx.CURSOR_ARROW))
        except:
            self.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))

        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)


        self.SetMinSize((size[1],size[0]))
    def SetImgSize(self, size):
        if isinstance(size, tuple):
            self.size = size

    def createGrid(self):
        try:
            color = cv2.cv.RGB(128, 128, 128)
        except:
            color = (128,128,128)
        
        grid = np.zeros_like(self.scaledImg,dtype=np.uint8)


        n = old_div(self.size[1],4)

        if n > 0:
            dx = float(grid.shape[0])/n
            for i in range(n+1):
                xi = int(i*dx)
                cv2.line(grid,(0,xi),(grid.shape[1],xi),color)


        n = old_div(self.size[0],8)

        if n > 0:
            dx = float(grid.shape[1]) / n
            for i in range(n + 1):
                xi = int(i * dx)
                cv2.line(grid, (xi, 0), (xi, grid.shape[0]), color)

        return grid

    def Draw(self, dc=None):
        """ Draw image """
        if dc is None:
            dc = wx.BufferedDC(wx.ClientDC(self), self.buffer)
        dc.DrawBitmap(self.bitmap, 0, 0)
        if self.startPos is not None:
            dc.SetPen(wx.Pen("white", 1))
            dc.SetBrush(wx.TRANSPARENT_BRUSH)
            try:
                dc.DrawRectangleRect(self.RectAround(self.startPos, self.endPos))
            except:
                dc.DrawRectangle(self.RectAround(self.startPos, self.endPos))

    def OnPaint(self, event=None):
        """ Process paint event """
        dc = wx.AutoBufferedPaintDC(self)
        self.Draw(dc)

    def OnSize(self, event=None):
        """ Process resize event """
        self.DCsize = tuple(self.GetClientSize())
        self.scaledImg = np.empty((self.DCsize[1], self.DCsize[0], 3), dtype=np.uint8)
        self.grid = self.createGrid()
        try:
            self.buffer = wx.Bitmap(*self.DCsize)
        except:
            self.buffer = wx.EmptyBitmap(*self.DCsize)
        self.Update(self.img)

    def Update(self, img):
#        self.img = img.transpose([1,0,2]).copy()
        self.img = img
        self.scaledImg = cv2.resize(
            self.img, self.DCsize, interpolation=cv2.INTER_NEAREST
        )
        if self.drawGrid:
            self.scaledImg = cv2.add(self.scaledImg, self.grid)
        try:
            self.bitmap = wx.Bitmap(
                wx.Image(*(self.DCsize + (bytearray(self.scaledImg.tostring()),)))
            )
        except:
            self.bitmap = wx.BitmapFromImage(
                wx.ImageFromData(
                    *(self.DCsize + (bytearray(self.scaledImg.tostring()),))
                )
            )

        wx.CallAfter(self.Draw, None)

    def OnLeftDown(self, evt):
        # Capture the mouse and save the starting posiiton for the
        # rubber-band
        self.CaptureMouse()
        self.startPos = self.endPos = self.PointToImgCoords(evt.GetPosition())
        self.Bind(wx.EVT_MOTION, self.OnMouseMove)
        evt.Skip()

    def OnMouseMove(self, evt):
        if evt.Dragging() and evt.LeftIsDown():
            p = self.PointToImgCoords(evt.GetPosition())
            if self.endPos != p:
                self.endPos = p

                evt.Skip()

                wx.PostEvent(self, UpdateEvent(False))
        else:
            evt.Skip()

    def OnLeftUp(self, evt):
        if self.HasCapture():
            self.ReleaseMouse()
        self.Unbind(wx.EVT_MOTION)
        self.Draw(None)
        evt.Skip()
        wx.PostEvent(self, UpdateEvent(True))

    def OnRightDown(self, evt):
        # restores the former content to the window.
        self.startPos = self.endPos = None
        self.Draw(None)
        evt.Skip()
        wx.PostEvent(self, UpdateEvent(True))

    def GetSelection(self):
        if self.startPos is None:
            return (None, None)
        x1, y1 = self.startPos
        x2, y2 = self.endPos

        return ( max(0,min(y1,y2)), max(0,min(x1,x2)) ), ( min(self.size[1]-1,max(y1,y2)), min(self.size[0]-1,max(x1,x2)) )

    def SetSelection(self,p1,p2=None):
        if isinstance(p1,tuple) and isinstance(p2,tuple): 
            self.startPos = ( max(0,min(p1[0],self.size[0]-1)), max(0,min(p1[1],self.size[1]-1)) )
            self.endPos  = ( max(0,min(p2[0],self.size[0]-1)),  max(0,min(p2[1],self.size[1]-1)) )
        elif isinstance(p1,tuple):
            self.startPos = ( max(0,min(p1[0],self.size[0]-1)), max(0,min(p1[1],self.size[1]-1)) )
            self.endPos = ( max(0,min(p1[0],self.size[0]-1)), max(0,min(p1[1],self.size[1]-1)) )

        elif p1 is None:
            self.startPos = self.endPos = None
        self.Draw(None)
        wx.PostEvent(self, UpdateEvent(True))

    def SelectAll(self):

        self.startPos = (0, 0)
        self.endPos = (self.size[0] - 1, self.size[1] - 1)
        self.Draw(None)
        wx.PostEvent(self, UpdateEvent(True))

    def PointToImgCoords(self, point):
        return (
            int(old_div(point[0] * self.size[0], self.DCsize[0])),
            int(old_div(point[1] * self.size[1], self.DCsize[1])),
        )

    def RectAround(self, p1, p2):
        x1 = int(old_div(max(0, min(p1[0], p2[0])) * self.DCsize[0], self.size[0]))
        x2 = int(
            old_div(
                min(self.size[0], max(p1[0], p2[0]) + 1) * self.DCsize[0], self.size[0]
            )
        )

        y1 = int(old_div(max(0, min(p1[1], p2[1])) * self.DCsize[1], self.size[1]))
        y2 = int(
            old_div(
                min(self.size[1], max(p1[1], p2[1]) + 1) * self.DCsize[1], self.size[1]
            )
        )

        return wx.Rect(x1, y1, x2 - x1, y2 - y1)


# End of class ImageDisplay(wx.Panel)


# GUI Frame class that spins off the worker thread
class MainFrame(wx.Frame):
    """Class MainFrame."""

    def __init__(self, parent, id, style=wx.DEFAULT_FRAME_STYLE):
        """Create the MainFrame."""

        self.flags = 0


        wx.Frame.__init__(
            self, parent, id, "Terasense View", style=style, size=(600, 600)
        )
        self.status = self.CreateStatusBar(number=4, style=wx.SB_RAISED)

        try:
            favicon = wx.Icon(
                os.path.join(
                    os.path.dirname(os.path.abspath(terasense.gui.__file__)), "tera.ico"
                ),
                wx.BITMAP_TYPE_ICO,
            )
            self.SetIcon(favicon)
        except:
            pass

        self.counter = 0

        self.fps = 0

        self.interface = None
        self.videoWriter = None
        self.selection = (None, None)  #
        self.values = {"mean": 0.0, "min": 0.0, "max": 0.0}  # mean, min, max

        self._stopme = False

        self.savePath = None
        # Create interface object

        try:
            defaults = __import__(
                "terasense.config", globals(), locals(), ["config"]
            ).config
        except:
            defaults = None

        try:
            config = os.path.join(os.path.dirname(__file__), defaults.configPath)
        except:
            config = None

        try:
            self.dev = defaults.developmentMode
        except:
            self.dev = False


        try:
            self.waterfall = defaults.waterfallMode
        except:
            self.waterfall = False
            

        try:
            self.processor = processor.processor(
                threaded=True, defaults=defaults, config=config, sn=commandline_options.serial
            )
        except RuntimeError as e:

            self.OnError(e)

            return

        self.status.SetStatusText(self.processor.interface.info(), 0)

        try:
            self.status.SetStatusText(self.processor.interface.get_id(), 1)
        except:
            pass

#<<<<<<< HEAD
#        size = (
#            int(round(self.processor.X_SIZE * pxSCALE * pxRATIO)),
#            int(round(self.processor.Y_SIZE * pxSCALE)),
#        )
#        self.imgSize = imgSize = (self.processor.X_SIZE, self.processor.Y_SIZE)
#
#        self.image = ImageDisplay(
#            self,
#            -1,
#            size=size,
#            style=wx.DOUBLE_BORDER | wx.RESIZE_BORDER,
#            imgSize=imgSize,
#        )
#        self.img = np.zeros(imgSize + (3,), dtype=np.uint8)
#        self.data = np.zeros(imgSize, dtype=np.float32)
#=======
        size = (int(round(self.processor.X_SIZE*pxSCALE*pxRATIO)),int(round(self.processor.Y_SIZE*pxSCALE)))
        imgSize = (self.processor.X_SIZE,self.processor.Y_SIZE)
        self.imgSize = (imgSize[1], imgSize[0])  ###
        
        self.image = ImageDisplay(self, -1, size = size, style = wx.DOUBLE_BORDER|wx.RESIZE_BORDER, imgSize = imgSize)
        self.img = np.zeros(imgSize+(3,),dtype=np.uint8)
        self.data = np.zeros(imgSize,dtype=np.float32)


        self.worker = worker.Worker(imgSize)


        self.nb = fnb.FlatNotebook(self, -1, agwStyle = fnb.FNB_NO_X_BUTTON | fnb.FNB_NO_NAV_BUTTONS )

        #
        self.SetBackgroundColour("#f0f0f0")

        self.saveimg = wx.Button(self, -1, "Save image...")
        self.saveimg.Disable()

        self.writevideo = wx.Button(self, -1, "Save video...")
        self.writevideo.Disable()

        
        self.startstop = wx.Button(self, -1, 'Start')

        if self.waterfall:
            self.wfBox = wx.StaticBoxSizer(wx.StaticBox(self), wx.HORIZONTAL)

            self.wfBox.AddSpacer(30)
            self.wf = wx.CheckBox(self, label="Waterfall", name="wf")
            self.Bind(wx.EVT_CHECKBOX, self.OnWaterfall, id=self.wf.GetId())
            self.wf.SetValue(False)
            self.wfBox.Add( self.wf, proportion=0, flag=wx.EXPAND|wx.ALIGN_CENTER)

            self.wfBox.AddSpacer(30)
            
            choices = self.worker.GetWFModes()
            self.wfMode = wx.RadioBox(self,label="Mode", choices=choices, name="wfMode",majorDimension=len(choices), style=wx.RA_SPECIFY_COLS)
            self.Bind(wx.EVT_RADIOBOX, self.OnWaterfallMode, id=self.wfMode.GetId())
            self.wfMode.SetSelection(0)
            self.wfBox.Add( self.wfMode, proportion=1, flag=wx.EXPAND|wx.ALIGN_CENTER)

        
        self.panelBCG = bcgpanel.BCGPanel(self.nb, -1, main = self, processor = self.processor)
        self.panelOptions = optionspanel.OptionsPanel(self.nb, -1,main = self, processor = self.processor)
        if  self.dev:

            try:
                import terasense.service.devpanel  as devpanel

                self.panelDev = devpanel.DevPanel(
                    self.nb, -1, main=self, processor=self.processor
                )
            except Exception as e:                
                self.panelDev = None
        else:
            self.panelDev = None

        if self.panelBCG:
            self.nb.AddPage(self.panelBCG, "B/C/G")
        if self.panelOptions:
            self.nb.AddPage(self.panelOptions, "Options")
        if self.panelDev:
            self.nb.AddPage(self.panelDev, "Debug")

        #  Menu
        self.menubar = wx.MenuBar()
        self.fileMenu = wx.Menu()
        mi = wx.MenuItem(
            self.fileMenu, wx.ID_SAVE, "&Save Image\tCtrl+S", "Save current image"
        )
        try:
            self.fileMenu.Append(mi)
        except:
            self.fileMenu.AppendItem(mi)
        self.Bind(wx.EVT_MENU, self.OnSaveImg, id=mi.GetId())


        mi = wx.MenuItem(self.fileMenu, -1, "&Save All\tF5", "Save all formats")
        try:
            self.fileMenu.Append(mi)
        except:
            self.fileMenu.AppendItem(mi)
        self.Bind(wx.EVT_MENU, self.OnSaveAll, id=mi.GetId())


        mi = wx.MenuItem(self.fileMenu, -1, "&Run\tCtrl+R", "Start")
        try:
            self.fileMenu.Append(mi)
        except:
            self.fileMenu.AppendItem(mi)
        self.Bind(wx.EVT_MENU, self.OnStart, id=mi.GetId())

        mi = wx.MenuItem(self.fileMenu, -1, "&Pause\tCtrl+P", "Stop")
        try:
            self.fileMenu.Append(mi)
        except:
            self.fileMenu.AppendItem(mi)
        self.Bind(wx.EVT_MENU, self.OnStop, id=mi.GetId())

        self.fileMenu.AppendSeparator()

        mi = wx.MenuItem(
            self.fileMenu, -1, "Select &All\tCtrl+A", "Selects entire area"
        )
        try:
            self.fileMenu.Append(mi)
        except:
            self.fileMenu.AppendItem(mi)
        self.Bind(wx.EVT_MENU, self.OnSelectAll, id=mi.GetId())

        self.fileMenu.AppendSeparator()

        mi = wx.MenuItem(self.fileMenu, wx.ID_EXIT, "&Quit\tCtrl+Q", "Quit application")
        try:
            self.fileMenu.Append(mi)
        except:
            self.fileMenu.AppendItem(mi)
        self.Bind(wx.EVT_MENU, self.OnQuit, id=mi.GetId())

        self.menubar.Append(self.fileMenu, "&File")

        self.helpMenu = wx.Menu()
        mi = wx.MenuItem(self.helpMenu, wx.ID_ABOUT, "&About...\tF10", "About the app")
        try:
            self.helpMenu.Append(mi)
        except:
            self.helpMenu.AppendItem(mi)
        self.Bind(wx.EVT_MENU, self.OnAboutBox, id=mi.GetId())

        self.menubar.Append(self.helpMenu, "&Help")

        self.SetMenuBar(self.menubar)

        #        self.Bind(wx.EVT_MENU, self.OnQuit, id=wx.ID_EXIT)

        self.statsBox = wx.StaticBox(self, label="Selection")
        self.statsData = wx.StaticText(self, label="0.000")
        font = self.statsData.GetFont()
        font.Scale(1.3)
        self.statsData.SetFont(font)
# <<<<<<< HEAD
        # self.statsData.SetMinSize(
            # self.statsData.GetTextExtent(
                # "Min: 0.000" + 4 * " " + "Max: 0.000" + 4 * " " + "Average: 0.000e-00"
            # )
        # )
        # ##        self.statsBox.Show(False)
        # ##        self.statsData.Show(False)

        # #  Assemble
        # sizer3 = wx.BoxSizer(wx.HORIZONTAL)

        # sizer3.Add(
            # self.image,
            # proportion=9,
            # flag=wx.SHAPED | wx.ALIGN_TOP | wx.ALL ^ wx.RIGHT,
            # border=0,
        # )
        # sizer3.Add(
            # self.nb, proportion=5, flag=wx.ALIGN_TOP | wx.LEFT | wx.RIGHT, border=10
        # )

        # self.mainSizer = wx.BoxSizer(wx.VERTICAL)

        # self.mainSizer.Add(
            # sizer3, proportion=3, flag=wx.ALIGN_CENTER | wx.EXPAND | wx.ALL, border=0
        # )

        # hbox = wx.BoxSizer(wx.HORIZONTAL)
        # ##        hbox = wx.StaticBoxSizer(wx.StaticBox(self), wx.HORIZONTAL)
        # self.statsSizer = wx.StaticBoxSizer(self.statsBox, wx.VERTICAL)
        # self.statsSizer.Add(
            # self.statsData,
            # proportion=0,
            # flag=wx.ALIGN_LEFT | wx.RIGHT | wx.LEFT,
            # border=5,
        # )
        # hbox.Add(self.statsSizer, proportion=0, flag=wx.ALIGN_CENTER_VERTICAL)
        # hbox.Add((1, 0), proportion=1)
        # hbox.Add(self.saveimg, proportion=0, flag=wx.EXPAND | wx.ALIGN_CENTER)
        # hbox.Add(self.writevideo, proportion=0, flag=wx.EXPAND | wx.ALIGN_CENTER)
        # hbox.Add((1, 0), proportion=1)
        # hbox.Add(self.startstop, proportion=0, flag=wx.EXPAND | wx.ALIGN_CENTER)
        # self.mainSizer.Add(
            # hbox, proportion=0, flag=wx.EXPAND | wx.ALIGN_RIGHT | wx.ALL, border=10
        # )
# =======
        self.statsData.SetMinSize(self.statsData.GetTextExtent("Min: 0.000"+4*" "+"Max: 0.000"+4*" "+"Average: 0.000e-00"))
##        self.statsBox.Show(False)
##        self.statsData.Show(False)

#  Assemble
        
        sizer3 = wx.BoxSizer(wx.VERTICAL)
        sizer3.Add( self.image,proportion=1, flag=wx.SHAPED|wx.ALIGN_CENTER|wx.ALL^wx.RIGHT,border=0)        
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        self.statsSizer = wx.StaticBoxSizer(self.statsBox,wx.VERTICAL)
        self.statsSizer.Add(self.statsData,proportion=0,flag=wx.ALIGN_LEFT|wx.RIGHT|wx.LEFT,border=5)
        hbox.Add(self.statsSizer,proportion=0,flag=wx.ALIGN_LEFT|wx.EXPAND)

       
        
        hbox.Add((1,0),proportion=1)
        hbox.Add(self.saveimg, proportion = 0,flag=wx.EXPAND|wx.ALIGN_CENTER)
        hbox.Add(self.writevideo, proportion = 0,flag=wx.EXPAND|wx.ALIGN_CENTER)

        if self.waterfall:
            hbox.Add((1,0),proportion=1)
            hbox.Add(self.wfBox, proportion = 0,flag=wx.EXPAND|wx.ALIGN_CENTER)

        hbox.Add((1,0),proportion=1)
        hbox.Add(self.startstop, proportion = 0,flag=wx.EXPAND|wx.ALIGN_CENTER)
        
        sizer3.Add(hbox,proportion=0, flag=wx.EXPAND|wx.ALIGN_RIGHT|wx.ALL,border=10)

        sizer1 = wx.BoxSizer( wx.HORIZONTAL)        
        
        sizer1.Add(sizer3,proportion=10, flag=wx.ALIGN_TOP|wx.EXPAND|wx.ALL,border=0)
        sizer1.Add( self.nb, proportion=2, flag=wx.ALIGN_TOP|wx.LEFT|wx.RIGHT,border=10)
        
        


        
        self.mainSizer  = sizer1
        self.SetSizerAndFit(self.mainSizer)

        self.mainSizer.SetSizeHints(self)

        self.currentSize = self.GetSize()

        self.Bind(wx.EVT_SIZE, self.OnSize)

        self.Bind(wx.EVT_BUTTON, self.OnSaveImg, id=self.saveimg.GetId())
        self.Bind(wx.EVT_BUTTON, self.OnWriteVideo, id=self.writevideo.GetId())

        self.Bind(wx.EVT_BUTTON, self.OnStart, id=self.startstop.GetId())


        self.Bind(wx.EVT_CLOSE, self.OnStop)




        # Set up event handler for any worker thread results
        self.Bind(EVT_UPDATE, self.OnUpdate)

        self.lastTime = 0

        self.SetWorkingArea()


    def OnAboutBox(self, event):

        description = """
"""
        licence = """
"""

        info = wx.adv.AboutDialogInfo()



        info.SetName('Terasense View (Linear)')
        info.SetVersion(version)
        info.SetDescription(description)
        info.SetCopyright("(C) 2012-2020 Terasense Development Labs")
        info.SetWebSite("http://www.terasense.com")
        info.SetLicence(licence)
        info.AddDeveloper("Gombo Tsydynzhapov")
        info.AddDeveloper("Oleg Volkov")

        wx.adv.AboutBox(info)


    def OnWaterfall(self, event=None):
        on = self.wf.IsChecked()
        dims1 = self.mainSizer.GetItem(0).GetSizer().GetItem(1).GetSize()
        dims = self.mainSizer.GetItem(0).GetSize()
        imgSize = self.worker.SetWaterfall(on, float(dims[1]-dims1[1])/dims[0])
        size = (int(round(imgSize[0]*pxSCALE*pxRATIO)),int(round(imgSize[1]*pxSCALE)))        

        szrItm = self.mainSizer.GetItem(0).GetSizer().GetItem(0)
        try:
            szrItm.SetRatio(size[1],size[0])
        except:
            szrItm.SetRatioWH(size[1],size[0])

        self.mainSizer.Layout()
        
        self.data = np.zeros(imgSize,dtype=np.float32)
        self.img = self.worker.makeImg(self.data)
        self.image.SetImgSize((imgSize[1],imgSize[0]))
        self.image.SetSelection(None)
        self.image.Update(self.img.copy())
        
        
        

    def OnWaterfallMode(self, event=None):
        self.worker.WaterfallMode(self.wfMode.GetStringSelection())
    

    def OnStart(self, event=None):
        """Start Computation."""

        self.counter = 0
        self.fps_list = []
        self.status.SetStatusText("Running...", 0)
        self.status.SetStatusText("", 1)
        self.status.SetStatusText("", 2)
        self.status.SetStatusText("", 3)

        self.startstop.SetLabel("Stop")
        self.Unbind(wx.EVT_BUTTON, id=self.startstop.GetId())
        self.Bind(wx.EVT_BUTTON, self.OnStop, id=self.startstop.GetId())

        #        self.Bind(worker.EVT_HISTOGRAM,self.panelBCG.OnStatData)

        self.saveimg.Enable()
        self.writevideo.Enable()

        if self.waterfall:
            self.wf.Disable()
            self.wfMode.Disable()

        try:
            self.panelBCG.OnStart()
        except:
            pass
        try:
            self.panelOptions.OnStart()
        except:
            pass
        try:
            self.panelWA.OnStart()
        except:
            pass
        try:
            self.panelDev.OnStart()
        except:
            pass

        self.processor.start(self.OnResult, self.OnError)
        self._stopme = False

        self.lastTime = time.perf_counter()


    def OnStop(self, event=None):
        """Stop Computation."""
        if isinstance(event, wx.CloseEvent):
            self.status.SetStatusText("Closing App...", 0)
            self.processor.stop(True)
        elif self.counter > 0:
            self.status.SetStatusText("Terminating...", 0)
            if not self._stopme:
                self._stopme = True  # Flag the worker thread to stop if running
        if event:
            event.Skip()

    def _OnStop(self, event=None):
        self.processor.stop()

        try:
            self.panelBCG.OnStop()
        except:
            pass
        try:
            self.panelOptions.OnStop()
        except:
            pass
        try:
            self.panelDev.OnStop()
        except:
            pass

        self.saveimg.Disable()  # disable screenshot
        self.writevideo.Disable()


        if self.waterfall:
            self.wf.Enable()
            self.wfMode.Enable()
            
            

        if self.counter > 0:
            self.status.SetStatusText("Stopped", 0)

        self.startstop.SetLabel("Start")
        self.Unbind(wx.EVT_BUTTON, id=self.startstop.GetId())
        self.Bind(wx.EVT_BUTTON, self.OnStart, id=self.startstop.GetId())

    def OnSelectAll(self, event=None):
        self.image.SelectAll()

    def OnQuit(self, event=None):
        self.Close()

    def OnSaveImg(self, event=None):
        if self.img is None:
            return
        im = self.img.copy()
        print(im.shape)
        w_data = self.worker.data.copy()
        print(w_data.shape)
        w_source = self.worker.source.copy()
        print(w_source.shape)
                
        dialog = wx.FileDialog(
            None,
            wildcard="PNG Image (*.png)|*.png|Image data (*.dat)|*.dat|Original data (*.dat)|*.dat",
            style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT,
        )
        if dialog.ShowModal() == wx.ID_OK:
            i = dialog.GetFilterIndex()
# <<<<<<< HEAD
            # if i == 0:
                # try:
                    # image = wx.Image(
                        # self.workingArea[1][0] + 1 - self.workingArea[0][0],
                        # self.workingArea[1][1] + 1 - self.workingArea[0][1],
                        # bytearray(self.img.tostring()),
                    # )
                # except:
                    # image = wx.ImageFromData(
                        # self.workingArea[1][0] + 1 - self.workingArea[0][0],
                        # self.workingArea[1][1] + 1 - self.workingArea[0][1],
                        # bytearray(self.img.tostring()),
                    # )
                # image.SaveFile(dialog.GetPath(), wx.BITMAP_TYPE_PNG)
            # else:
                # if i == 1:
                    # x = self.worker.data[
                        # self.workingArea[0][0] : self.workingArea[1][0] + 1,
                        # self.workingArea[0][1] : self.workingArea[1][1] + 1,
                    # ].copy()
                    # comment = "Processed image data"
                    # str_format = "%3d"
                # elif i == 2:
                    # x = self.data[
                        # self.workingArea[0][0] : self.workingArea[1][0] + 1,
                        # self.workingArea[0][1] : self.workingArea[1][1] + 1,
                    # ].copy()
                    # comment = "Original image data"
                    # str_format = "%.3f"
                # with open(dialog.GetPath(), "w") as out:
                    # out.write("# " + comment + "\n")
                    # out.write(
                        # "# %dx%d\n"
                        # % (
                            # self.workingArea[1][0] + 1 - self.workingArea[0][0],
                            # self.workingArea[1][1] + 1 - self.workingArea[0][1],
                        # )
                    # )
# =======
            if i==0:
                im.transpose([1,0,2])
                image = wx.ImageFromData(*(im.shape[1::-1]+(bytearray(im.tostring()),)))
                image.SaveFile(dialog.GetPath(),wx.BITMAP_TYPE_PNG)
            else:
                if i == 1:
                    x = np.transpose(w_data)
                    comment = "Processed image data"

                    str_format = "%3d"
                elif i == 2:
                    x = np.transpose(w_source)

                    comment = "Original image data"
                    str_format = "%.5f"
                with open(dialog.GetPath(),"w") as out:
                    out.write('# '+comment+'\n')
                    out.write('# %dx%d\n' % x.shape)
# >>>>>>> Linear-detector
                    for i in range(x.shape[1]):
                        out.write(",\t".join(str_format % a for a in x[:, i]) + "\n")

        dialog.Destroy()


    def OnSaveAll(self, event=None):
        if self.img is None:
            return
        im = self.img.copy()
        w_data = self.worker.data.copy()
        w_source = self.worker.source.copy()
        try:
            stl = wx.FD_SAVE | wx.FD_CHANGE_DIR
        except:
            stl = wx.SAVE | wx.FD_CHANGE_DIR
        if self.savePath is None:
            dialog = wx.FileDialog(None, wildcard="PNG|*.png|Data| *.dat", style=stl)
            if dialog.ShowModal() == wx.ID_OK:
                self.savePath = dialog.GetDirectory()
                dialog.Destroy()
            else:
                dialog.Destroy()
                return

        dirlist = os.listdir(self.savePath)
        cnt = -1
        for name in dirlist:
            s = re.match("^" + IMG_PREFIX + "(\d\d\d)\.", name)
            if s is not None:
                cnt = max(cnt, int(s.group(1)))

        name = IMG_PREFIX + "{:03d}".format(cnt + 1)

        try:
            image = wx.Image(im.shape[1], im.shape[0], bytearray(im.tostring()))
        except:
            image = wx.ImageFromData(im.shape[1], im.shape[0], bytearray(im.tostring()))
        image.SaveFile(os.path.join(self.savePath, name + ".PNG"), wx.BITMAP_TYPE_PNG)

        ##                    x = np.transpose(self.worker.data).copy()
        x = np.transpose(w_data)
        comment = "Processed image data"
        with open(os.path.join(self.savePath, name + ".dat"), "w") as out:
            out.write("# " + comment + "\n")
            out.write("# %dx%d\n" % x.shape)
            for i in range(x.shape[1]):
                out.write(",\t".join("%3d" % a for a in x[:, i]) + "\n")

        ##                    x = np.transpose(self.data).copy()
        x = np.transpose(w_source)
        comment = "Original image data"
        with open(os.path.join(self.savePath, name + ".orig.dat"), "w") as out:
            out.write("# " + comment + "\n")
            out.write("# %dx%d\n" % x.shape)
            for i in range(x.shape[1]):
                out.write(
                    ",\t".join("%.5f" % a for a in x[:, i]) + "\n"
                )

##    def OnSaveAll(self, event=None):
##        if  self.img is None:
##            return
##        im = self.img.copy()
##        w_data = self.worker.data.copy()
##        data = self.data.copy()
##        try:
##            stl = wx.FD_SAVE|wx.FD_CHANGE_DIR
##        except:
##            stl = wx.SAVE|wx.FD_CHANGE_DIR
##        if self.savePath is None:
##            dialog = wx.FileDialog ( None, wildcard = "PNG|*.png|Data| *.dat", style = stl  )
##            if dialog.ShowModal() == wx.ID_OK:
##                self.savePath = dialog.GetDirectory()
##                dialog.Destroy()
##            else:
##                dialog.Destroy()
##                return
##
##        dirlist = os.listdir(self.savePath)
##        cnt =-1
##        for name in dirlist:
##            s = re.match('^'+IMG_PREFIX+'(\d\d\d)\.',name)
##            if s is not None:
##                cnt = max(cnt, int(s.group(1)))
##
##        name = IMG_PREFIX+'{:03d}'.format(cnt+1)
##                    
##        try:
##            image = wx.Image(im.shape[1], im.shape[0], bytearray(im.tostring()))
##        except:
##            image = wx.ImageFromData(im.shape[1], im.shape[0], bytearray(im.tostring()))
##        image.SaveFile(os.path.join(self.savePath, name+".PNG"),wx.BITMAP_TYPE_PNG)
##
##    ##                    x = np.transpose(self.worker.data).copy()
##        x = w_data
##        comment = "Processed image data"
##        with open(os.path.join(self.savePath, name+".dat"),"w") as out:
##            out.write('# '+comment+'\n')
##            out.write('# %dx%d\n' % x.shape)
##            for i in range(x.shape[1]):
##                out.write(",\t".join("%3d" % a for a in x[:,i])+"\n")
##                    
##    ##                    x = np.transpose(self.data).copy()
##        x = data
##        comment = "Original image data"
##        with open(os.path.join(self.savePath, name+".orig.dat"),"w") as out:
##            out.write('# '+comment+'\n')
##            out.write('# %dx%d\n' % x.shape)
##            for i in range(x.shape[1]):
##                out.write(",\t".join("%.5f" % a for a in x[:,x.shape[1]-i-1])+"\n")




    def OnWriteVideo(self, event=None):
        fps = int(self.fps)
        if self.videoWriter is not None:
            if not nosound:
                winsound.Beep(600, 300)
            self.writevideo.SetLabel("Save video...")
            self.status.SetStatusText("", 1)
            self.saveimg.Enable()
            self.startstop.Enable()
            del self.videoWriter
            self.videoWriter = None
        else:
            dialog = wx.FileDialog(
                None,
                wildcard="MPEG4 P2 Video (*.avi)|*.avi|Uncompressed AVI (*.avi)|*.avi",
                style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT,
            )
            if dialog.ShowModal() == wx.ID_OK:

                if min(self.img.shape[0:2]) < MIN_VIDEO_OUT:
                    k = float(MIN_VIDEO_OUT)/min(self.img.shape[0:2])
                    self.videoout_size =  (int(k * self.img.shape[1]),int(k * self.img.shape[0]))
                    frame_size = self.videoout_size
                else:
                    self.videoout_size = None
                    frame_size = self.img.shape[1::-1]
                path = os.path.abspath(dialog.GetPath())
                if dialog.GetFilterIndex()==0:
                    try:                                        #opencv 3.0
                        fcc =cv2.CV_FOURCC('X','V','I','D')

                    except:
                        fcc = cv2.VideoWriter_fourcc("X", "V", "I", "D")
                else:
                    fcc = 0

                try:
                    self.videoWriter = cv2.VideoWriter(
                        filename=path,
                        fourcc=fcc,
                        fps=fps if fps > 1 else 1,
                        frameSize=frame_size,
                        isColor=1,
                    )
                except:
                    self.videoWriter = cv2.VideoWriter(
                        filename=path.encode("utf-8"),
                        fourcc=fcc,
                        fps=fps if fps > 1 else 1,
                        frameSize=frame_size,
                        isColor=1,
                    )

                self.writevideo.SetLabel("Stop")
                self.status.SetStatusText("Recording videostream", 1)
                self.saveimg.Disable()
                self.startstop.Disable()
            dialog.Destroy()
            if not nosound:
                winsound.Beep(1000, 300)

    def OnUpdate(self, event=None):
        #        self.image.Update(self.worker.img)

        self.selection = self.image.GetSelection()

        if (self.workingArea is None) or (self.selection[0] is None):
            p1, p2 = self.selection
        else:
            p1 = (
                self.selection[0][0] + self.workingArea[0][0],
                self.selection[0][1] + self.workingArea[0][1],
            )
            p2 = (
                self.selection[1][0] + self.workingArea[0][0],
                self.selection[1][1] + self.workingArea[0][1],
            )

        if event is None or event.data:
            self.values = self.worker.statistics((p1, p2))
            if p1 is None:
                self.statsData.SetLabel("")
                self.statsBox.SetLabel("")
            elif p1 == p2:
                self.statsData.SetLabel("%(mean).3f" % self.values)
                self.statsBox.SetLabel("Selection: {}, {}".format(p1[1],p1[0]))
            else:

                self.statsData.SetLabel(("Min: %(min).3f"+4*" "+"Max: %(max).3f"+4*" "+"Average: %(mean).3e") % self.values )

                self.statsBox.SetLabel("Selection: {0}:{2}, {1}:{3}".format(p1[1],p1[0],p2[1],p2[0]))
            self.status.SetStatusText("frame #%d" % self.counter,2)
            self.status.SetStatusText( "%.2f" % self.fps +" fps",3)

        else:
            if p1 is None:
                self.statsData.SetLabel("")
                self.statsBox.SetLabel("")
            elif p1 == p2:
                self.statsData.SetLabel("%(mean).3f" % {"mean": 0.0})
                self.statsBox.SetLabel("Selection: {}, {}".format(*p1))
            else:
                self.statsData.SetLabel(
                    (
                        "Min: %(min).3f"
                        + 4 * " "
                        + "Max: %(max).3f"
                        + 4 * " "
                        + "Average: %(mean).3e"
                    )
                    % {"min": 0.0, "max": 0.0, "mean": 0.0}
                )
                self.statsBox.SetLabel("Selection: {0}:{2}, {1}:{3}".format(*(p1 + p2)))


    def OnResult(self, data):
        """Show Result status."""

        self.data = data.copy()

        self.img = self.worker.makeImg(data)
        self.counter += 1
        
        newtime = time.perf_counter()
        if len(self.fps_list)<11 :
            self.fps_list.append( 1./(newtime-self.lastTime) )
            self.fps = sorted(self.fps_list)[old_div((len(self.fps_list) - 1), 2)]

        else:
            self.fps_list.append(1.0 / (newtime - self.lastTime))
            self.fps_list.pop(0)

            self.fps = sorted(self.fps_list)[2]

        try:
            wx.MutexGuiEnter()
            self.image.Update(self.img.copy())
            wx.MutexGuiLeave()
        except:
            wx.CallAfter(self.image.Update, self.img.copy())

        if self._stopme:
            wx.CallAfter(self._OnStop)
            self._stopme = False
            return
        else:
            if self.fps < 60 or (self.counter % 3 == 0):
                wx.PostEvent(self, UpdateEvent(True))


        self.lastTime = newtime
        if self.videoWriter is not None and self.videoWriter.isOpened():
 #           im = self.img.copy()
            if self.videoout_size is not None:

                self.videoout_frame = cv2.resize(
                    self.img, self.videoout_size, interpolation=cv2.INTER_NEAREST
                )
            else:
                self.videoout_frame = self.img.copy()

            cv2.cvtColor(self.videoout_frame, cv2.COLOR_RGB2BGR, self.videoout_frame)
            self.videoWriter.write(self.videoout_frame)

    def OnError(self, event=None, critical=True):
        """Hardware error"""

        dialog = wx.MessageDialog(
            None,
            "Runtime Error:\n"
            + str(event)
            + ("\n\nApplication will be closed." if critical else ""),
            "Terasense View",
            wx.OK | wx.ICON_ERROR,
        )
        dialog.ShowModal()
        dialog.Destroy()
        if critical:
            self.Close()
        return

    def OnSize(self, event=None):
        """ Process resize event - keep aspect ratio"""

        size = self.GetSize()
        if event:
            event.Skip()
            return
            if abs(self.currentSize[0] - size[0]) > abs(self.currentSize[1] - size[1]):
                newsize = (
                    size[0],
                    int(
                        old_div(
                            float(size[0]) * self.currentSize[1], self.currentSize[0]
                        )
                    ),
                )
            else:
                newsize = (
                    int(
                        old_div(
                            float(size[1]) * self.currentSize[0], self.currentSize[1]
                        )
                    ),
                    size[1],
                )
            self.SetSize(newsize)
            self.currentSize = newsize
            event.Skip()

    def SetWorkingArea(self, rect=None):

        try:
            self.workingArea = (
                (
                    max(0, min(rect[0][0], rect[1][0])),
                    max(0, min(rect[0][1], rect[1][1])),
                ),
                (
                    min(self.imgSize[0] - 1, max(rect[0][0], rect[1][0])),
                    min(self.imgSize[1] - 1, max(rect[0][1], rect[1][1])),
                ),
            )
        except:
            self.workingArea = ((0, 0), (self.imgSize[0] - 1, self.imgSize[1] - 1))

        dims1 = self.mainSizer.GetItem(0).GetSizer().GetItem(0).GetSize()
        dims = self.mainSizer.GetItem(0).GetSize()

        imgSize = (
            self.workingArea[1][0] - self.workingArea[0][0] + 1,
            self.workingArea[1][1] - self.workingArea[0][1] + 1,
        )

        size = (
            int(round(imgSize[0] * pxSCALE * pxRATIO)),
            int(round(imgSize[1] * pxSCALE)),
        )

        szrItm = self.mainSizer.GetItem(0).GetSizer().GetItem(0)
        try:
            szrItm.SetRatio(*size)
        except:
            szrItm.SetRatioWH(*size)
        self.mainSizer.Layout()

        self.processor.SetROI(self.workingArea)
        self.worker.SetROI(self.workingArea)

        ##        self.data = np.zeros(imgSize,dtype=np.float32)
        self.img = self.worker.makeImg(self.data)[
            self.workingArea[0][0] : self.workingArea[1][0] + 1,
            self.workingArea[0][1] : self.workingArea[1][1] + 1,
        ].transpose([1, 0, 2])
        self.image.SetImgSize(
            (
                self.workingArea[1][0] + 1 - self.workingArea[0][0],
                self.workingArea[1][1] + 1 - self.workingArea[0][1],
            )
        )
        self.image.Update(self.img.copy())


# Main wxPython App
class MainApp(wx.App):
    """Class Main App."""

    def OnInit(self):
        """Init Main App."""
        self.instance = wx.SingleInstanceChecker("TeraSense Viewer App 3")
        if not commandline_options.multihead:
            if self.instance.IsAnotherRunning():
                wx.MessageBox("ERROR - Another instance is running", "TeraSense Viewer")
                return False
        self.frame = MainFrame(None, -1, style=wx.DEFAULT_FRAME_STYLE)
        self.frame.Show(True)
        self.SetTopWindow(self.frame)
        return True


if __name__ == "__main__":
    version = '3.5.3'#version - do not delete this comment!
    parser = OptionParser(version=version)
    parser.add_option("-m", "--multi",
                      action="store_true", dest="multihead", help="allow multiple instances for multihead operation", default=False)
    parser.add_option("-s", "--serial",
                      action="store", type='string', dest="serial", help="try to open device with the given serial number", default='')
    parser.add_option("-l", "--list",
                      action="store_true", dest="list", help="list serial numbers of available devices and exit", default=False)
    parser.add_option("-t", "--test",
                      action="store_true", dest="testmode", help="run in a test mode, without hardware", default=False)    
    (commandline_options, args) = parser.parse_args()    
    if commandline_options.list:
        de = processor.deviceEnumerator()
        for s in de.getDevList():
            print(s)
    else:
        if commandline_options.testmode:
            from terasense import iface as iface
            iface.test = True
            
        app = MainApp(0)
        app.MainLoop()
        wx.Exit()
