#!/usr/bin/env python3

from __future__ import print_function
from builtins import str
from builtins import range
__svn_revision__ = '$Revision: 291 $'

# Get data frame from TS32A

import sys
import getopt
import numpy as np

import terasense.config.pixel_config as pc



from terasense.lib.log import *
from terasense.lib.common import *


opt_n = 1

def inf(s):
        if opt_n <= 1:
                msg(l_inf, s)

def notice(s):
        if opt_n <= 1:
                msg(l_notice, s)

#log.inf = inf
#log.notice = notice

usage =\
"""
TS32A single shot acquisition tool.
Usage:
  ts_get.py [options]
Options available:
  -V          show version  
  -x x1,x2,x3:x4       x coordinates
  -y y1,y2,y3:y4       y coordinates
  -e exp           choose exposure (default 3)
  -r            output raw data (no background compensation, no normalization)
  -n count		number of repetitions for averaging		
  -h          this help
The output consists of data for all (x,y) combinations. There is as many lines as y coordinates and as many
comma-separated data points per line as x coordinates.
See tbus_conf.py, ts_conf.py and ts32a_conf.py, pixel_conf.py for more configuration options.
"""

def main():

        global opt_n
        opt_v = False

        try:
                optlist, args = getopt.getopt(sys.argv[1:], 'Vx:y:e:n:rh')
        except getopt.GetoptError:
                err('invalid command line options' + ': ' + repr(sys.argv[1:]))
                return -1


        xlist=None
        ylist=None
        raw = False
        exp = 3
        count = 1
        for opt in optlist:
                if opt[0] == '-V':
                        print("Revision:", revision, type(revision))
                        inf(tools_title + ' ' + ver_title + '.' + str(revision.tool_revision('ts_get')))
                        opt_v = True
                elif opt[0] == '-x':
                        strings = opt[1].split(",")
                        xlist = []
                        for s in iter(strings):
                                try:
                                        if ":" in s:
                                                (start,end)=s.split(":",1)
                                                xlist.extend(list(range(int(start),int(end)+1)))
                                        else:
                                                xlist.append(int(s))
                                except ValueError :
                                        print("Invalid value %s for command line option %s." % (s,opt[0]), file=sys.stderr)
                                        return -1
                elif opt[0] == '-y':
                        strings = opt[1].split(",")
                        ylist = []
                        for s in iter(strings):
                                try:
                                        if ":" in s:
                                                (start,end)=s.split(":",1)
                                                ylist.extend(list(range(int(start),int(end)+1)))
                                        else:
                                                ylist.append(int(s))
                                except ValueError :
                                        print( "Invalid value %s for command line option %s." % (s,opt[0]), file=sys.stderr)
                                        return 1
                elif opt[0] == '-e':
                        exp = int(opt[1])
                elif opt[0] == '-r':
                        raw = True
                elif opt[0] == '-r':
                        exp = int(opt[1])
                elif opt[0] == '-n':
                        count = int(opt[1])
                elif opt[0] == '-h':
                        print(usage)
                        return 1
        
       

        try:
                if raw:
                        
                        from terasense import iface as iface
                        print('iface:')
                        source = iface.interface(threaded=False)
                else:
                        from terasense import processor as iface
                        print('Proccessor:')
                        source = iface.processor(threaded=False)
        except RuntimeError:
                print( "RuntimeError: Controller is not connected or busy (another copy running?)" , file=sys.stderr)
                return 1
        source.SetExposure(exp)
        print("Not Final data:")
        data = source.read()
        print("Final data:", data)
        for i in range(count-1):
                np.add(data,source.read(),data)
        np.divide(data,count,data)
        if not xlist:
                xlist=list(range(len(data)))
        if not ylist:
                ylist=list(range(len(data[0])))
        for y in ylist:
                print( ",".join([str(data[x][y]) for x in xlist]))
        return 0

if __name__ == '__main__':
        
#        import cProfile
#        cProfile.run('main()','stats')
        main()
