from __future__ import division
from past.utils import old_div
import wx
import random, math

# Default level of black
DEFAULT_BLACK=0

# GUI BCG panel class
class BCGPanel(wx.Panel):
    """"Class BCGPanel"""
    def __init__(self,parent,id, main=None,processor = None):

        super(BCGPanel, self).__init__(parent=parent,id=id)

        if not main:
            self.main = parent.GetParent()
        else:
            self.main = main
            
        self.processor = processor
        
        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add( (0,1),proportion=1, flag=wx.EXPAND)

        

        vbox.Add(wx.StaticText(self,label="White")) 
        self.white = wx.Slider(self, style=wx.SL_TOP |wx.SL_AUTOTICKS |wx.SL_LABELS, value=100)
        self.white.SetTickFreq(50)
        self.Bind(wx.EVT_SLIDER, self.OnChangeWhite, id=self.white.GetId())
        vbox.Add( self.white, proportion=1, flag=wx.EXPAND|wx.ALIGN_CENTER)
        

        vbox.Add(wx.StaticText(self,label="Black")) 
        self.black = wx.Slider(self, style=wx.SL_TOP |wx.SL_AUTOTICKS | wx.SL_LABELS, value=DEFAULT_BLACK)
        self.black.SetTickFreq(50)
        self.Bind(wx.EVT_SLIDER, self.OnChangeBlack, id=self.black.GetId())
        vbox.Add( self.black, proportion=1, flag=wx.EXPAND|wx.ALIGN_CENTER)
        

        
        vbox.Add(wx.StaticText(self,label="Gamma"))
        self.gamma = wx.Slider(self, style=wx.SL_INVERSE | wx.SL_TOP |wx.SL_AUTOTICKS | wx.SL_LABELS, value=50)
        self.gamma.SetTickFreq(50)
        self.Bind(wx.EVT_SLIDER, self.OnChangeGamma, id=self.gamma.GetId())
        vbox.Add( self.gamma, proportion=1, flag=wx.EXPAND|wx.ALIGN_CENTER)

        self.exposureLabel = wx.StaticText(self,label="Exposure")
        vbox.Add(self.exposureLabel)
        minExp,maxExp = self.processor.interface.GetExposureRange()        
        self.exposure = wx.Slider(self, style=wx.SL_TOP |wx.SL_AUTOTICKS | wx.SL_LABELS, minValue=minExp, maxValue=maxExp,value=self.processor.GetExposure())
        # value range from 0 to 6 (2^16 usec frame to 2^10 usec)
        self.exposure.SetTickFreq(1)
        self.Bind(wx.EVT_SLIDER, self.OnChangeExposure, id=self.exposure.GetId())
        vbox.Add( self.exposure, proportion=1, flag=wx.EXPAND|wx.ALIGN_CENTER)

        


        
        hbox = wx.BoxSizer(wx.HORIZONTAL)
##        self.autoexp = wx.Button(self,-1, 'Autoexp')
##        self.Bind(wx.EVT_BUTTON, self.OnAutoexp, id=self.autoexp.GetId())
##        self.autoexp.Disable()
##        hbox.Add( self.autoexp, proportion=1)
##        self.autolevel = wx.Button(self,-1, 'Autolevel')
##        self.Bind(wx.EVT_BUTTON, self.OnAutolevel, id=self.autolevel.GetId())
##        self.autolevel.Disable()
##        hbox.Add( self.autolevel, proportion=1)
        self.resetlevel = wx.Button(self,-1, 'Reset')
        self.Bind(wx.EVT_BUTTON, self.OnResetLevel, id=self.resetlevel.GetId())
        hbox.Add((1,0),proportion=1)
        hbox.Add( self.resetlevel, proportion=0, flag=wx.ALIGN_CENTER_VERTICAL)
        vbox.Add( hbox,proportion=1, flag=wx.EXPAND)
        
        vbox.Add( (0,1),proportion=1, flag=wx.EXPAND|wx.ALIGN_CENTER)

        self.SetSizerAndFit(vbox)

        

        
    def Initialize(self):    
        self.OnChangeWhite()
        self.OnChangeBlack()
        self.OnChangeGamma()
##        self.OnChangeExposure()

    def OnStart(self):
##        self.autolevel.Enable()
##        self.autoexp.Enable()
        self.Initialize()
        
    def OnStop(self):
##        self.autolevel.Disable()
##        self.autoexp.Disable()
        pass

    
    def controlFuncBW(self,x):
        return (float(x)/100)**2
    def invFuncBW(self,x):
        val = int(float(x)**(0.5)*100)
        return (val if val<100 else 100) if val > 0 else 0
    
    def OnChangeWhite(self,event=None):        
        white = float(self.white.GetValue())
        black = float(self.black.GetValue())
        if white < black:
            black = white-1
            self.black.SetValue(black)

        self.main.worker.SetBrightness(self.controlFuncBW(black),self.controlFuncBW(white))
        self.main.worker.SetContrast(self.controlFuncBW(black),self.controlFuncBW(white))
                
    def OnChangeBlack(self,event=None):
        white = float(self.white.GetValue())
        black = float(self.black.GetValue())
        if white < black:
            white = black +1
            self.white.SetValue(white)
        self.main.worker.SetBrightness(self.controlFuncBW(black),self.controlFuncBW(white))
        self.main.worker.SetContrast(self.controlFuncBW(black),self.controlFuncBW(white))

    def controlFuncGamma(self,val):
        val = float((val if val<100 else 100) if val > 0 else 0)
        return math.tan(old_div((0.86+old_div((val-50),80))*math.pi,4))
    def invFuncGamma(self,x):
        val = int(math.atan(old_div(x*4,math.pi)-0.85)*80+50)
        return (val if val<100 else 100) if val > 0 else 0
    
    def OnChangeGamma(self,event=None):

        val = self.gamma.GetValue()
        self.main.worker.SetGamma(self.controlFuncGamma(val))

    def OnChangeExposure(self,event=None):
   #     oldTime = self.interface.GetIntTime()
        e = self.exposure.GetValue()     
        if (not event) or self.processor.GetExposure() != e:
            self.exposure.Disable()
            err=self.processor.SetExposure(e)
            if err:
                self.main.OnError(err)
            self.main.fps_list = []
            wx.CallAfter(self.exposure.Enable)



##    def OnAutoexp(self,event=None):
##        if not self.stats or self.stats['max'] <= 0:
##            return
##
##        Sum = sum(self.stats['bins'])
##        test=self.stats['bins'][0]
##        i=0
##        while test<Sum*0.02:
##            i+=1
##            test+=self.stats['bins'][i]
##            
##        p02=i           
##        while test<Sum*0.5:
##            i+=1
##            test+=self.stats['bins'][i]
##        median=i
##        while test<Sum*0.98:
##            i+=1
##            test+=self.stats['bins'][i]
##        p98 = i
##
##        
##        expRange = self.processor.interface.GetExposureRange()
##        e = self.processor.interface.GetExposure()
##        if p98+1 == len(self.stats['bins']):
##            if e > expRange[0]:
##                newexp = e-1
##        else:
##            wantTime = self.processor.interface.GetIntTime()/self.stats['max']
##            newexp = e+1
##            while self.processor.interface.GetIntTime(newexp) < wantTime :
##                
##                newexp +=1
##                if newexp > expRange[1]:
##                    break
##            newexp -=1
##        self.exposure.SetValue(newexp)
##        self.OnChangeExposure()
##
##        self.OnResetLevel()
##
##    def OnAutolevel(self,event=None):
###        clearance = 0.05*(self.stats['max']-self.stats['min'])
###        white = int((self.stats['max']-clearance)*100)
###        black = int((self.stats['min']+clearance)*100)
###        gamma = 1
##        bottomRange = 0.02
##        topRange = 0.98
##        if not self.stats:
##            return
##        Sum = sum(self.stats['bins'])
##        test=self.stats['bins'][0]
##        i=0
##        while test<Sum*bottomRange:
##            i+=1
##            test+=self.stats['bins'][i]
##            
##        bottom=i           
##        while test<Sum*0.5:
##            i+=1
##            test+=self.stats['bins'][i]
##        median=i
##        while test<Sum*topRange:
##            i+=1
##            test+=self.stats['bins'][i]
##        top = i
##        black = float(bottom)/len(self.stats['bins'])
##        white = float(top+1)/len(self.stats['bins'])
##        
##        x = (float(median)/len(self.stats['bins'])-black)/(white-black)
##        if x <= 0:
##            gamma = 1
##        elif math.log(x) == 0:
##            gamma = 10
##        else: 
##            gamma = math.log(0.3)/math.log(x)        
##        
##        
##        self.black.SetValue(self.invFuncBW(black))
##        self.OnChangeBlack()
##
##        self.white.SetValue(self.invFuncBW(white))
##        self.OnChangeWhite()
##        
##        self.gamma.SetValue(self.invFuncGamma(gamma))
##        self.OnChangeGamma()


    def OnResetLevel(self,event=None):
        self.white.SetValue(100)
        self.OnChangeWhite()
        self.black.SetValue(DEFAULT_BLACK)
        self.OnChangeBlack()
        self.gamma.SetValue(50)
        self.OnChangeGamma()
