__svn_revision__ = '$Revision: 433 $'

#
# TS32 data to image conversion routine
#

import numpy as np
import config.ts_conf as ts_conf

pconf = __import__(ts_conf.pix_config, fromlist=[''])
pmap = None

def build_pixels_map(pixels):
	"""Build module / channel to pixel mapping"""
	global pmap
	sensor_height = pixels / pconf.pixels_in_row

	assert pixels > 0
	assert pixels % pconf.pixels_in_row == 0
	assert sensor_height % pconf.mod_h == 0

	m_ind = np.empty((sensor_height, pconf.pixels_in_row), dtype=int)
	c_ind = np.empty((sensor_height, pconf.pixels_in_row), dtype=int)

	for r in range(sensor_height):
		for c in range(pconf.pixels_in_row):
			m_ind[r, c], (mr, mc) = pconf.mod_resolver(r, c, pixels)
			assert mr <= r and mc <= c
			c_ind[r, c] = pconf.mod_pixels[r - mr][c - mc]

	if pconf.image_fliplr:
		m_ind, c_ind = np.fliplr(m_ind), np.fliplr(c_ind)
	if pconf.image_flipud:
		m_ind, c_ind = np.flipud(m_ind), np.flipud(c_ind)
	if pconf.image_transpose:
		m_ind, c_ind = np.transpose(m_ind), np.transpose(c_ind)

	pmap = (m_ind, c_ind)

def get_image_pixels(data):
	"""Convert data array to image array according to pixels config"""
	pixels = data.shape[0] * pconf.mod_w * pconf.mod_h
	if pmap is None:
		build_pixels_map(pixels)
	else:
		assert pmap[0].size == pixels and pmap[1].size == pixels

	return ts_conf.channel_data(data, pmap[0], pmap[1])
