from __future__ import print_function
from __future__ import absolute_import
from builtins import chr
from builtins import str
from builtins import range
__svn_revision__ = '$Revision: 433 $'

# T-BUS control tool

import sys
import traceback
import getopt
import random

from . import config.path

from . import tbus_if
from . import config.tbus_conf as tbus_conf
import terasense.lib.log as log

from terasense.lib.log import *
from terasense.lib.common import *
from .tmod_hlp import *

from . import tbus_defs as tbus
from . import tmod_defs as tmod

opt_n = 1
opt_tbus_cmd = None

def inf(msg):
	if opt_n <= 1:
		log.inf(msg)

def query_ctl_id():
	ctl = tbus_if.tbus_ctl.create(tbus_conf)
	if not ctl.init_core():
		err('%s is not connected' % ctl.info())
		return -1

	inf(ctl.get_id())
	return 0

def set_ctl_id(idstr):
	ctl = tbus_if.tbus_ctl.create(tbus_conf)
	if not ctl.init_core():
		err('%s is not connected' % ctl.info())
		return -1

	ctl.set_id(idstr)
	return 0

def query_modules_info(ctl):
	r = read_status(ctl)
	inf('\n%d module(s):' % r[0])
	inf('version             = 0x%x' % r[1][0])
	inf('hardware id         = 0x%x' % r[1][1])
	inf('hard reset          = %s' % ((r[1][2] & tmod.hard_reset) != 0))
	inf('clock uncalibrated  = %s' % ((r[1][2] & tmod.clock_uncalibrated) != 0))
	inf('srq pending         = %s' % ((r[1][2] & tmod.srq_pending) != 0))
	inf('chksum valid        = %s' % ((r[1][2] & tmod.chksum_valid) != 0))
	inf('bus error           = %s' % ((r[1][2] & tmod.tbus_err) != 0))
	inf('system error        = %s' % ((r[1][2] & tmod.sys_err) != 0))
	inf('srq status          = %#x' % r[1][3])
	inf('code buffer check sum %08x' % r[1][4])
	return 0

def query_temp(ctl):
	dbg('reading modules temperature')
	t_raw = ctl.bus_read_struct(tmod.temp_addr, 'H')
	t_K   = [tmod.get_abs_temp(v[0]) for v in t_raw if v != 0]
	if not len(t_K):
		err('modules temperature is not available yet')
		return -1
	inf('%d module(s) T = %d .. %dC (%u .. %u)' % (len(t_K), min(t_K)-273, max(t_K)-273, min(t_raw)[0], max(t_raw)[0]))
	return 0

def memory_test(ctl):
	# Generate random data
	length = 1 + random.getrandbits(tmod.srq_buff_chunk_bits)
	data = ''
	# Put random content to SRQ buffer
	for i in range(length):
		data += chr(random.getrandbits(8))
	dbg('writing %d bytes to SRQ buffer' % length)
	ctl.bus_write(tmod.srq_buff_addr, data)
	dbg('resetting modules')
	ctl.bus_reset()
	dbg('polling SRQ buffer content')
	r = ctl.bus_read_coherent(tmod.srq_buff_addr, length)
	# Check content
	if r[1] != data:
		err('modules memory content mismatch')
		return -1
	return 0

def tbus_test(ctl):
	# Generate random data
	addr   = random.getrandbits(16)
	length = 1 + random.getrandbits(tbus.data_length_bits)
	data = ''
	for i in range(length):
		data += chr(random.getrandbits(8))
	dbg('passing %d bytes over the bus' % length)
	r = ctl.bus_cmd(tbus.cmd_test, addr, data)
	dbg('%d responses' % r)	
	return 0

def flash_test(ctl):
	# Generate random data
	length = 1 + random.getrandbits(tmod.seg_bits)
	data = ''
	for i in range(length):
		data += chr(random.getrandbits(8))
	dbg('writing %d bytes to the flash' % length)
	# Write single segment at address valid on all hardware
	program(ctl, 0xf800, data)

def erase_flash(ctl):
	dbg('erasing flash')
	srq_sync(ctl, tmod.srq_flash_ers)
	return 0

def send_tbus_cmd(ctl):
	assert opt_tbus_cmd is not None
	dbg('sending %s command' % tbus.cmd_names[opt_tbus_cmd])
	r = ctl.bus_cmd(opt_tbus_cmd)
	inf("T-BUS %s -> %d responses" % (tbus.cmd_names[opt_tbus_cmd], r))
	return 0

def bus_srq(ctl):
	dbg('issuing SRQ on the bus')
	ctl.bus_srq()

def bus_rst(ctl):
	dbg('issuing RESET on the bus')
	ctl.bus_reset()

def main():
	global opt_n, opt_tbus_cmd
	opt_handler = None
	opt_err_continue = False
	force_fw_load = False

	try:
		optlist, args = getopt.getopt(sys.argv[1:], 'iI:n:cmTMEsrwetfFvh')
	except getopt.GetoptError:
		err('invalid command line options' + ': ' + repr(sys.argv[1:]))
		return -1

	if args:
		err('unexpected args: %s' % args)
		return -1

	for opt, arg in optlist:
		if opt == '-n':
			opt_n = int(arg)
		elif opt == '-c':
			opt_err_continue = True
		elif opt == '-i':
			return query_ctl_id()
		elif opt == '-I':
			return set_ctl_id(arg)
		elif opt == '-m':
			opt_handler = query_modules_info
		elif opt == '-T':
			opt_handler = query_temp
		elif opt == '-M':
			opt_handler = memory_test
		elif opt == '-E':
			opt_handler = erase_flash
		elif opt == '-s':
			opt_handler = bus_srq
		elif opt == '-r':
			opt_handler = bus_rst
		elif opt == '-w':
			opt_handler = send_tbus_cmd
			opt_tbus_cmd = tbus.cmd_wait
		elif opt == '-e':
			opt_handler = send_tbus_cmd
			opt_tbus_cmd = tbus.cmd_enum
		elif opt == '-t':
			opt_handler = tbus_test
		elif opt == '-f':
			opt_handler = flash_test
		elif opt == '-F':
			force_fw_load = True
		elif opt == '-v':
			log.verbose += 1
		elif opt == '-h':
			print("""
T-BUS debug tool. Print controller info by default. Options available:
  -i          query controller ID
  -I          set   controller ID
  -m          query modules info
  -T          query temperature
  -M          modules memory test
  -E          erase code buffer
  -s          issue SRQ on the bus
  -r          issue RESET on the bus
  -w          issue WAIT command on the bus
  -e          issue ENUM command on the bus
  -t          issue TEST command with random data
  -f          flash writing test
  -n N        repeat N times
  -c          continue after error (for N > 1)
  -F          force load firmware
  -v          verbose output
  -h          this help
See tbus_conf.py for more configuration options.
""")
			return 0

	if not opt_handler:
		inf(tools_title + ' ' + ver_title + '.' + str(revision))

	ctl = tbus_if.tbus_ctl.create(tbus_conf)
	if not ctl.init(force_load=force_fw_load):
		err('%s is not connected' % ctl.info())
		return -1

	if not opt_handler:
		inf(ctl.info(True))
		return 0

	if opt_n != 1:
		if opt_err_continue:
			print('looping %d times, errors ignored ...' % opt_n)
		else:
			print('looping %d times ...' % opt_n)

	for i in range(opt_n):
		try:
			res = opt_handler(ctl)
			if res:
				return res
		except KeyboardInterrupt:
			raise
		except:
			if not opt_err_continue:
				raise
			if log.verbose:
				traceback.print_exc()

	log.inf('\nok')
	return 0

if __name__ == '__main__':
	main()
