from __future__ import absolute_import
from builtins import range
from builtins import object
from importlib import import_module
__svn_revision__ = '$Revision: 433 $'

# 
# T-BUS controller abstract interface
#

import struct
from . import tbus_defs as tbus
# from . import tbus_ok as tbus_ok

class tbus_ctl(object):
	def __init__(self, cfg):
		"""The cfg parameter receive the implementation specific parameters namespace"""
		self.cfg = cfg

	@staticmethod
	def create(cfg):
		"""Create instance according to the configuration passed"""
		# impl = __import__(cfg.controller, globals(),locals())
		# Hardcode below
		impl = __import__(cfg.controller, globals(),locals(), fromlist=['tbus_ok'])
		# impl = import_module("terasense.tbus.tbus_ok")
#		print(type(impl.tbus_ctl(cfg)))
		# End of hardcode
		return impl.tbus_ctl(cfg)

	def init(self):
		"""Initialize controller. Returns True on success, False on failure."""
		raise NotImplementedError

	def init_core(self):
		"""Initialize with limited functionality (implementation specific)"""
		return self.init()

	def get_id(self):
		"""Returns id string of the controller hardware"""
		raise NotImplementedError

	def set_id(self, idstring):
		"""Sets id string of the controller hardware"""
		raise NotImplementedError

	def info(self, detailed=False):
		"""Returns string describing the controller hardware"""
		raise NotImplementedError

	def bus_reset(self):
		"""Issue hard reset on the bus"""
		raise NotImplementedError

	def bus_srq(self):
		"""Issue SRQ on the bus"""
		raise NotImplementedError

	def bus_cmd(self, cmd_code, addr = 0, data = None):
		"""
		Send single response command. Raise exception if the command status is not success
		or the data received differs from the data sent. Returns the number of responses.
		"""
		raise NotImplementedError

	def bus_read_raw(self, addr, data_len):
		"""
		Send read command. Raise exception if the command status is not success.
		Returns the (r_cnt, data) tuple.
		"""
		raise NotImplementedError

	def bus_read(self, addr, data_len):
		"""Read data from the specified address. Returns the array of the data responses."""
		r = self.bus_read_raw(addr, data_len)
		return [r[1][i*data_len:(i+1)*data_len] for i in range(r[0])]

	def bus_read_struct(self, addr, fmt):
		"""
		Read data from the specified address and parse it according to specified format.
		Returns the array of the data tuples.
		"""
		r = self.bus_read(addr, struct.calcsize(fmt))
		return [struct.unpack(fmt, d) for d in r]

	def bus_read_coherent(self, addr, data_len):
		"""
		Read coherent data from the specified address. Raise exception if the command status
		is not success or the data is not coherent. Returns (r_cnt, data) tuple.
		"""
		raise NotImplementedError

	def bus_read_struct_coherent(self, addr, fmt):
		"""
		Read coherent data from the specified address and parse it according to specified format
		Returns (r_cnt, data_tuple) tuple.
		"""
		r = self.bus_read_coherent(addr, struct.calcsize(fmt))
		return (r[0], struct.unpack(fmt, r[1]))

	def bus_write(self, addr, data):
		"""Write data to the specified address"""
		if not self.bus_cmd(tbus.cmd_write, addr, data):
			raise RuntimeError('No target modules for write')

	def set_wait_tout(self, msec_tout):
		"""Set wait timeout in milliseconds"""
		raise NotImplementedError

	def bus_wait(self):
		"""Issue wait command"""
		if not self.bus_cmd(tbus.cmd_wait):
			raise RuntimeError('No target modules for wait')
