from __future__ import division
from __future__ import absolute_import
from past.utils import old_div
__svn_revision__ = '$Revision: 433 $'

# TS32 utilities

import struct
import serial

from . import tbus_if
from .config import tbus_conf as tbus_conf
from . import tmod_defs as tmod
import terasense.lib.msp430txt as ldr
import terasense.lib.log as log

from terasense.lib.log import *
from .tmod_hlp import *

import numpy as np
import os

data_channels = 35
data_len = data_channels * 2
max_data_value = 1024
timeout_margin = 1000

def exp_time(conf):
	"""Return estimated exposure time in msec"""
	frame = 1 << (16 - conf.timer_frame)
	frame_usec = old_div((frame << conf.timer_clk_div), 4)
	return old_div((data_channels * frame_usec), 1000)

def configure(ctl, conf):
	"""Reset modules, load applet if necessary and configure timeout"""

	if hasattr(conf, 'on_load'):
		conf.on_load(ctl)

	path = os.path.join(os.path.dirname(__file__),conf.code_path)
	log.dbg('using code path %s' % path)
#	print("Path:", path)
	segs = ldr.load_file(path)
	if segs is None:
		raise RuntimeError("failed to load %s" % path)
	if len(segs) != 1:
		raise RuntimeError("can't load %d segments from %s" % (len(segs), path))

#	print("Segs[0]:", segs[0], type(segs))
	addr, code = segs[0]
	conf._start_address = addr
	chksum = tmod.get_chksum(code)
	notice('applet is loaded: @%x %d bytes, chksum=%08x' % (addr, len(code), chksum))

	notice('resetting the bus')
	reset(ctl)

	notice('queering modules status')
	try:
		r = read_status(ctl)
	except RuntimeError:
		notice('erasing flash')
		srq_sync(ctl, tmod.srq_flash_ers)
		r = read_status(ctl)

	if not (r[1][2] & tmod.chksum_valid):
		raise RuntimeError('no check sum reported')

	if r[1][4] != chksum:
		inf('loading applet to flash')
		program(ctl, addr, code)
	else:
		notice('applet is already loaded to flash')

def acquire_raw(ctl, conf):
	"""Execute applet and returns (number of module responses, raw data string) tuple"""
	tout = exp_time(conf) + timeout_margin
	notice('setting wait timeout to %d msec' % tout)
	ctl.set_wait_tout(tout)
	notice('starting applet ..')
	srq_sync(ctl, tmod.srq_proc, 0, conf._start_address, struct.pack(
			'HBB',
			conf.int_time,
			conf.timer_frame | (conf.reset_time << 3) | (conf.timer_clk_div << 6),
			conf.adc_sh_time | (conf.adc_clk_div << 2 ) | (conf.no_cdc << 5) | (conf.temp_test << 6) | (conf.vcc_ref << 7)
		), conf.read_delay)
	notice("reading results ..")
#	print("Data_len:", data_len)
	r = ctl.bus_read_raw(tmod.srq_buff_addr + 4, data_len)
#	print("Acquire_raw:", r)
	return r

def array_from_raw(data):
	"""Create numpy array from raw data"""
	a = np.fromstring(data[1], dtype=np.int16)
	assert a.size == data_channels * data[0]
	if np.any(a >= max_data_value):
		raise RuntimeError('the data is out of valid range')
	return np.reshape(a, (old_div(a.size,data_channels), data_channels))

def acquire(ctl, conf):
	"""Execute applet and returns results as numpy array"""
	data = acquire_raw(ctl, conf)
	return array_from_raw(data)

def acquire_auto(ctl, cb, conf):
	"""Run srq/read cycle in auto mode calling supplied callback on each iteration"""
	def proxy(data):
		return cb(array_from_raw(data))

	if not cb(array_from_raw(acquire_raw(ctl, conf))):
                ctl.bus_read_auto(data_len, proxy)
